/*****************************************************************************
 * File: MultiSignalReader.hpp
 * Author: Jonathan Gips, MIT Wearable Computing Group
 * http://www.media.mit.edu/wearables/
 * Copyright (C) 2002 MIT Media Lab
 *
 * Adapted from OpineViewerEngine.hpp
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************/
#ifndef MULTISIGNALREADER_HPP
#define MULTISIGNALREADER_HPP

#include <stdlib.h>
#include <getopt.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <string.h>

extern "C" {
#include <enchantment/whiteboard.h>
#include <enchantment/signals.h>
#include <enchantment/time-utils.h>
#include <enchantment/unl.h>
#include "terse.h"
}


#define USE_THREADS

#ifdef USE_THREADS
#include <pthread.h>
#include <semaphore.h>
#endif

#define CERT_FILE "cert.pem"
#define KEY_FILE "key.pem"

#ifdef USE_THREADS
pthread_mutex_t fileLock;
pthread_mutex_t ipcLock;
#define lock(name) pthread_mutex_lock(&name)
#define unlock(name) pthread_mutex_unlock(&name)
#else
#define lock(name)
#define unlock(name)
#endif


class MultiSignalReader
{
protected:
  static const unsigned int kFeatureBufferLen = 16384;
  
  typedef struct {
    pthread_mutex_t recLock;
    Signal *sig;
    char *sigName;
    bool foundInUpdate;
    bool destroy;
  } MultiSignalRec;

  typedef struct {
    MultiSignalReader *reader;
    MultiSignalRec *rec;
  } ThreadRec;
  
  char *subUNL;
  FILE *outFD;
  bool terse;
  
  bool lockMethod;
  pthread_mutex_t printLock;

  DArray recs; //holds the MultiSignalRecs
  
  // Used for traversal
  static int ProcessUpdateNode(void *reader, NodeType *processNode, 
			       NodeType *parentNode, const char *UNL);
  
  static void *receiveSignal(void *data);
  static void *doExec(void *data);

  
public:
  void handleUpdate(NodeType *updateNode, const char *unl, const char *publisher);
  int lockFile();
  int unlockFile();

  MultiSignalReader(const char *subUNL, FILE* outFD, bool terse);
  virtual ~MultiSignalReader();

  
};

#endif // MULTISIGNALREADER_HPP
