    /*************************************************
 * File: SignalViewer.c
 * Author: unknown
 * Birth Date: 2/28/03 
 * Last update: 5/26/05 by Mark Blum:  introduced a terse output form which 
 *   can be directly imported into Matlab
 *  
 * 
 * Copyright (C) 2002-2005 MIT Media Lab
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 *
 *************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/time.h>
#include <signal.h>
#include <math.h>
#include <strings.h>

#include <termios.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/ioctl.h>

#include <enchantment/whiteboard.h>
#include <enchantment/signals.h>
#include <enchantment/time-utils.h>
#include <enchantment/unl.h>
#include "terse.h"

void PrintUsage()
{
    //this is the same syntax as used in MultiSignalReader
    printf("Usage:MultiSignalSignalReader -s [context server name] -p [signal branch/node path] [-t]\n");
    printf("Default options: -s <localhost>\n");
    printf("( -t results in terse output)\n");
}

void SigHandler(int sig) {
    switch (sig) {
        case SIGINT:
            fflush(stdout);
            fprintf(stderr,"Hard kill, flushing stdout and exiting.\n");
            exit(0);
            break;    
        default:
            fprintf(stderr,"Signal handler received %d\n",sig);
    }
    return;
}

int main(int argc, char *argv[])
{
    Signal *inSignal;
    char UNL[128];
    char *buffer = NULL;
    int i;
    char *server = NULL, *path = NULL;
    int serverSet = 0;
    int pathSet = 0;
    int proxy = 0;
    int terse = 0;

    //Die nicely
    signal(SIGINT,SigHandler);
       
    for(i = 1; i < argc; i++)
    {
        if(argv[i][0] == '-')
        {
            if((argv[i][1] == 's'))
            {
                i++;
                server = argv[i];
                serverSet = 1;
            }
            else if((argv[i][1] == 'p'))
            {
                i++;
                path = argv[i];
                pathSet = 1;
            }
            else if((argv[i][1] == 't'))
            {
                terse = 1;
            }
            else if((argv[i][1] == 'x'))
            {
                proxy = 1;
            }
            else if((argv[i][1] == 'h'))
            {
                PrintUsage();	      
                exit(1);
            }
        } 
    }

    if(!pathSet)
    {
        printf("Path argument missing!\n");
        PrintUsage();
        exit(1);
    }
  
    if(!serverSet) 
    {
        server = "localhost";
    }
        
    sprintf(UNL,"%s:%s",server,path);
    inSignal = NewSignalUNL(UNL);
    if (!terse) printf("SignalViewer: viewing signal at %s\n", UNL);
        
    while (1)
    {
        struct timeval time;
        if (!SignalConnected(inSignal))
        {
	  if (proxy ? !SignalConnectByProxy(inSignal) : 
                     !SignalConnectWait(inSignal))
          {
              printf("Failed to connect to signal\n");
              exit(1);
          }
          if (SignalConnected(inSignal))
          {
              if (!terse) printf("Connected\n");
              if (buffer)
                  free(buffer);
              buffer = malloc(SignalBufferSize(inSignal));
          }
        }
        else
        {
            if (SignalRecv(inSignal, &time, buffer))
            {
                
                if (!terse) 
                {
                    PrintSignalElements(stdout, inSignal, time, buffer); //this is the call
		  //to the built in method in signals.c
                }
                else 
                {
                    TerselyPrintSignalElements(stdout, inSignal, time, buffer); //in terse.h
		}
	    }
        }
    }
}
