/*****************************************************************************
 * File: EnchantRunning.c
 * Author: Anmol Madan, MIT Wearable Computing Group
 * Adapted from ench.c, by Daniel Barkalow
 * http://www.media.mit.edu/wearables/
 * Copyright (C) 2003 MIT Media Lab
 *
 * This program is meant to be used in conjuntion with the zaurus_adhoc_update.sh 
 * script. The script identifies alive IP addresses from a file containing the list 
 * of possible adhoc addresses, and then calls this program to verify if they are really 
 * running  Enchantment servers. 
 *
 * Return Type 0 = running enchanment, 1 = Not Running, 2 = Error in commandline parameters
 *
 * Set Timeout duration using TIMEOUT
 *
 * IMP Note : This code is meant to be used only for valid IP address, tested with fping 
 * It may be confused by Invalid IPs
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************/
//add functionality to make timeout settable from commandline


#include <unistd.h>    /* standard unix functions, like alarm()          */
#include <signal.h>    /* signal name macros, and the signal() prototype */
#include <stdlib.h>
#include <stdio.h>
#include <sys/time.h>

#include <enchantment/node.h>
#include <enchantment/whiteboard.h>
#include <enchantment/unl.h>
#include <enchantment/dstring.h>


#define TIMEOUT 500

void SignalHandler(int);

int main(int argc, char *argv[])
{
  int loopTime=TIMEOUT;
  char *UNL, *mach;
  struct itimerval *value;
  NodeType *node = NULL;
  GetNodeInfo getInfo;

  if(argv[1]==NULL)
    {
      printf("\n Usage : enchRunning [machine name/ip] \n"); 
      exit(2);
    }
  mach = argv[1];
  
  value = (struct itimerval *)(malloc(sizeof(struct itimerval)));
  //  loopTime *= 1000; //convert to micro sec
 
  //Setup signal handler
   signal(SIGALRM,SignalHandler);
  
   // Set timer value and reset value
   value->it_interval.tv_sec=0;
   value->it_interval.tv_usec=loopTime*1000;
   value->it_value.tv_sec=0;
   value->it_value.tv_usec=loopTime*1000;

   UNL = CreateUNL(mach, "/clients/Server");
   
   // Set real-time timer so that we can get un-stuck
   setitimer(ITIMER_REAL,value,NULL);
   
   // Try to get a node
   node = GetNode(UNL, &getInfo);
   
   if(node)
     {
       exit(0);
     }
   else
     {
       exit(1);
     }
}


/* define an alarm signal handler. */
void SignalHandler(int sig_num)
{
  //Operation timed out. Enchantment not up on ..
  exit(1);
}













