/*****************************************************************************
*
* Author: Steve Dunn, MIT Wearable Computing Group
* http://www.media.mit.edu/wearables/
* Copyright (C) 2002 MIT Media Lab
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
*****************************************************************************/

#ifndef GAUSSIAN_HPP
#define GAUSSIAN_HPP

#include "XMLObject.hpp"

extern "C" {
#include <enchantment/vector.h>
#include <enchantment/matrix.h>
}

class Gaussian : public XMLObject {
protected:
  XMLVector *meanXML;
  XMLMatrix *covXML;
  XMLMatrix *invcovXML;

public:
  int identifier;
  char *label;
  unsigned int dimension;
  double sigma_det;
  Vector *u;
  Matrix *cov;
  Matrix *sigma_inv;
  
  int _Verbose;
  
  typedef enum XMLState {GAUSSIAN, LABEL, DET, MEAN, 
			 COV, INVCOV, SKIP} XMLState;
  XMLState xml_state;
  
  Gaussian(unsigned int dimension = 0);
  virtual ~Gaussian();
  
  virtual double calc(Vector *feature);
  virtual void show();
  virtual void clear();
  
  virtual void startXMLElement(const char *name, const char **atts); 
  virtual void processXMLCharacterData(const char *s, int len);
  virtual void endXMLElement(const char *name);
};

#endif

