#ifndef XMLOBJECT_HPP
#define XMLOBJECT_HPP

extern "C" {
#include <enchantment/vector.h>
#include <enchantment/matrix.h>
}

#include "expat.h"

class XMLObject {
private:
  static const int XML_BUFF_SIZE;
  XML_Parser parser;
  XMLObject *parentObj;
  
protected:
  virtual XML_Parser getParser() { return parser; }
  virtual void setParser(XML_Parser pars) { parser = pars; }
  
  virtual void setParent(XMLObject *parent) { parentObj = parent; }
  virtual XMLObject *getParent() { return parentObj; }

  

  virtual void pushXMLObject(XMLObject *obj, const char *name, const char **atts);
  virtual void popXMLObject();

  static void StartXMLElementWrapper(void *thisObj, const char *name, const char **atts);
  static void EndXMLElementWrapper(void *thisObj, const char *name);
  static void XMLCharacterDataHandlerWrapper(void *thisObj, const char *s, int len);
  static void StartXMLElementNOP(void *thisObj, const char *name, const char **atts) {}
  static void EndXMLElementNOP(void *thisObj, const char *name) {}
  static void XMLCharacterDataHandlerNOP(void *thisObj, const char *s, int len) {}

  virtual void startXMLElement(const char *name, const char **atts) {};
  virtual void endXMLElement(const char *name) {};
  virtual void processXMLCharacterData(const char *s, int len) {};
  
  static const char *skipCharacterDataWS(const char *s);

  static char *MyStrndup(const char *s, int len);

public:
  XMLObject();
  virtual ~XMLObject();

  virtual int loadXML(char *filename);
  virtual void show() { printf("XMLOBject\n"); }
};  

class XMLVector : public XMLObject {
protected:
  typedef enum XMLState { VECTOR, SKIP };
  XMLState xml_state;

public:
  unsigned int dim;
  Vector *v;
  
  XMLVector(unsigned int dim = 0);  
  virtual ~XMLVector();

  void clear();
  
  virtual void startXMLElement(const char *name, const char **atts);
  virtual void endXMLElement(const char *name);
  virtual void processXMLCharacterData(const char *s, int len); 
  
  virtual void show() { 
    if(v != NULL) {
      PrintVector(stdout, v);
    } else printf("NULL XMLVector\n");
  }
};

class XMLMatrix : public XMLObject {
protected:
  typedef enum XMLState { MATRIX, NUMROWS, NUMCOLS, ROW, SKIP };
  XMLState xml_state;
  
  int rowIdx;

public:
  unsigned int row;
  unsigned int column;
  Matrix *m;
  
  XMLMatrix(unsigned int row = 0, unsigned int column = 0);
  virtual ~XMLMatrix();

  void clear();
  static bool GetTranspose(Matrix *m, double *trans);

  virtual void startXMLElement(const char *name, const char **atts);
  virtual void endXMLElement(const char *name);
  virtual void processXMLCharacterData(const char *s, int len);
  
  virtual void show() { 
    if(m != NULL) {
      PrintMatrix(stdout, m);
    } else printf("NULL XMLMatrix\n");
  }
};

#endif
