/*************************************************
 * File: linux_audio.h
 * Author: unknown
 * Birth Date: unknown
 * Modified by: Mark Blum (mark@media.mit.edu)
 *
 * Project: ASSIST
 * 
 * 
 * Copyright (C) 2005 MIT Media Lab
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 ************************************************
 * This file is originally from the 'inference' modul. It is included
 * in the SignalRecorder application only for the possibility to play
 * sound on request.
 ***********************************************/

#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h> 
#include <sys/stat.h> 
#include <fcntl.h> 
#include <sys/ioctl.h> 
#include <linux/soundcard.h>
#include <unistd.h> /* close() */
#include <string.h> /* memset() */
				/* for some reason this doesn't seem
                                   to be defined??? */


#include <enchantment/signals.h>


char *strndup(const char *s, size_t size);

/* #define DEBUG - used by IR in lkt_serial.c*/
/* #define TCP_NODELAY_MODE */
/* #define TCP_NONBLOCK_MODE */

/* Global storage for speed, channels, and format information */

extern char *linux_audio_rdevice;
extern int linux_audio_rspeed;
extern int linux_audio_rchannels;
extern int linux_audio_rformat;
extern int linux_audio_rdescript;

extern char *linux_audio_wdevice;
extern int linux_audio_wspeed;
extern int linux_audio_wchannels;
extern int linux_audio_wformat;
extern int linux_audio_wdescript;

/* Default buffer size for audio read/write transactions */

#define DEFAULT_AUDIO_BUFFER_SIZE 1024

/* Platform-independent defaults for audio device configuration */

#define DEFAULT_AUDIO_SPEED 11025
#define DEFAULT_AUDIO_CHANNELS 1
#define DEFAULT_AUDIO_FORMAT AFMT_S16_LE

/* Platform-dependent defaults for audio device configuration */

#ifdef ZAURUS
#define DEFAULT_INPUT_AUDIO_DEVICE "/dev/dsp"
#define DEFAULT_OUTPUT_AUDIO_DEVICE "/dev/dsp"
#define AUDIO_CONFIG "zaurus"
#else
#define DEFAULT_INPUT_AUDIO_DEVICE "/dev/dsp"
#define DEFAULT_OUTPUT_AUDIO_DEVICE "/dev/dsp"
#define AUDIO_CONFIG "desktop"
#endif


/* Configures the linux audio device for output.

   Default parameter values are used if arguments are NULL or zero.

   Returns 1 on success, zero on failure.
   File descriptor is stored in linux_audio_wdescript.  
*/

void linux_audio_close();

int linux_init_write_sound(char *device, /* device name */
			   int speed,    /* sampling rate */
			   int channels, /* mono or stereo */
			   int format);  /* ioctl data format magic */

/* Configures the linux audio device for output.

   Uses the configuration information in a cSignal for some
   configuration parameters, if *signal is non-null.

   Default parameter values are used if arguments are NULL or zero.

   Returns 1 on success, zero on failure.
   File descriptor is stored in linux_audio_wdescript.  
*/

int linux_init_write_sound_signal(Signal *signal, /* cSignal */
				  char *device,
				  int format);

/* Configures the linux audio device for input.

   Default parameter values are used if arguments are NULL or zero.

   Returns 1 on success, zero on failure.
   File descriptor is stored in linux_audio_rdescript.  
*/

int linux_init_read_sound(char *device, /* device name */
			  int *speed,    /* sampling rate */
			  int *channels, /* mono or stereo */
			  int format);  /* ioctl data format magic */

/* This plays back raw wave recorded at 22050, 16 bit, mono */ 
/* Does this function work or make sense in this context?  What if the
   audio settings don't match?  Shouldn't the configuration be read
   from the WAV header? */

int linux_play_wave(char *filename);

/* This function reads audio samples (using the current settings) into
   the signed short buffer pointed to by *buffer, of length size.  It
   returns the number of samples read.  */

int linux_audio_read(signed short *buffer, int size);

/* This function plays audio (using the current settings) from the
   signed short buffer pointed to by *buffer, of length size.  It
   returns the number of samples played. */

int linux_audio_play(signed short *buffer, int size);

