/*************************************************
 * File: SignalRateTester.c
 * Birth Date: Feb 03, 2005
 *
 * Project: Enchantment IPC
 * 
 * Based on Enchantment IPC Whitepaper
 *   by Edward Keyes.
 *
 * Copyright (C) 2005 AWare Technologies, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *************************************************/

#include <stdlib.h>

#include <enchantment/signals.h>
#include <enchantment/time-utils.h>

int main(int argc, char **argv)
{
  Signal *sig;
  char *buffer = NULL;
  struct timeval stamp, prev, current;
  struct timeval first;
  unsigned count = 0;
  double avgrate;
  long diff;
  long max, min;
  long off;
  sig = NewSignalUNL(argv[1]);
  printf("-Count --Offset-- --Rate-- --Diff:     Max    Min\n");
  printf("               us     1/us      us      us     us\n");
  do
    {
      if (!SignalConnected(sig))
	{
	  SignalConnectWait(sig);
	  if (buffer)
	    free(buffer);
	  buffer = malloc(SignalBufferSize(sig));
	  count = 0;
	  continue;
	}
      SignalRecv(sig, &stamp, buffer);
      count++;
      if (count == 1)
	{
	  first = stamp;
	  prev = stamp;
	}
      else
	{
	  diff = timeval_sub(&stamp, &prev);
	  gettimeofday(&current, NULL);
	  off = timeval_sub(&stamp, &current);
	  if (count == 2 || diff > max)
	    max = diff;
	  if (count == 2 || diff < min)
	    min = diff;
	  prev = stamp;
	  avgrate = (double) timeval_sub(&stamp, &first) / (count - 1);
	  printf("%6d %10ld %8d %6ld %6ld %6ld\n", 
		 count, off, (int) avgrate, diff, max, min);
	}
    }
  while (1);
}
