/*************************************************
 * File: ipc.h
 * Author: Jonathan Gips
 * Birth Date: 6/27/01
 *
 * Project: Enchantment IPC
 * 
 * Based on Enchantment IPC Whitepaper
 *   by Edward Keyes.
 *
 * Copyright (C) 2002 MIT Media Lab
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 *
 *************************************************/

#ifndef IPC_H
#define IPC_H

#include <enchantment/node.h>
#include "socket.h"

typedef NodeType *(*IPCNodeProc)(void *, NodeType *);

int     SerializeNode(Socket *sock, NodeType *node, void *data, 
		      IPCNodeProc proc);
int     DeserializeNode(Socket *sock, NodeType **p_node, void *data, 
			IPCNodeProc proc);

void    PrintSockStats();

void ReadString(Socket *socket, char **result, size_t *size_p);
void ReadType(Socket *socket, TypeID *type);
void ReadValue(Socket *socket, TypeID type, EnchantData *value);
void ReadNode(Socket *socket, NodeType **node_p, IPCNodeProc proc, void *data);

void WriteString(Socket *socket, const char *value, size_t length);
void WriteType(Socket *socket, TypeID type);
void WriteValue(Socket *socket, TypeID type, EnchantData value);
void WriteNode(Socket *socket, NodeType *node, IPCNodeProc proc, void *data);

void ntoh(char *buffer, size_t ineltsize, size_t outeltsize, 
	  unsigned length, TypeID type);

void convcopy(char *in, char *out, size_t ineltsize, size_t outeltsize, 
	      unsigned length, TypeID type);

#endif // IPC_H
