/*************************************************
 * File: vector.h
 * Birth Date: Aug 18, 2004
 *
 * Project: Enchantment IPC
 * 
 * Based on Enchantment IPC Whitepaper
 *   by Edward Keyes.
 *
 * Copyright (C) 2004 Aware Technologies, Inc.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 * 
 *************************************************/

#ifndef ENCHANTMENT_VECTOR_H
#define ENCHANTMENT_VECTOR_H

#include <stdio.h>

typedef struct
{
  double *values;
  unsigned dimension;
} Vector;

/* Creates a new vector of the given length. */
Vector *NewVector(unsigned length);

/* Copies a vector. */
Vector *CopyVector(Vector *v);

/* Returns the value at the specified (0-based) location in the vector. */
static __inline__ double VectorRef(Vector *V, unsigned idx)
{
  return V->values[idx];
}

/* Sets the value at the specified (0-based) location in the vector. */
static __inline__ void VectorSet(Vector *V, unsigned idx, double value)
{
  V->values[idx] = value;
}

/* Reads a vector from a file of whitespace-separated values. */
Vector *ReadVector(FILE *fp, unsigned length);

/* Subtracts two vectors, returning a new vector. */
Vector *VectorSubtract(Vector *a, Vector *b);

/* Writes a vector to a file in a human-readable format. */
void PrintVector(FILE *fp, Vector *v);

#endif
